-- 首充
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local rechargemod = require "model.rechargemod"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local activitytimemod
local first_recharge2
local first_recharge3

local MAX_DAY = 3

local module_name = "first_recharge1"
local module_id = MODULE_ID_FIRST_RECHARGE1
local _M = schema.new(module_name, {
    state = false,  -- 是否购买
    start_time = 0, -- 购买的时间
    day = 1,
    award = 0,  -- 已领取的奖励
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN} -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_unlock(character)
        end
    end)
    character.monitor("daily_refresh",function()
        local d = _M.assert_get(character)
        if d.state then
            d.day = d.day + 1
            _M.persist(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
    first_recharge2 = require "model.activity.first_recharge2"
    first_recharge3 = require "model.activity.first_recharge3"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end


function THIS.first_recharge_buy(character)
    local d = _M.assert_get(character)
    local conf = get_conf(character)
    if not conf.gift[1] then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    
    local gift_conf = asset.GiftConfig_proto[conf.gift[1]]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if d.state then
        return STD_ERR.COMMON_HAVE_GIFT -- 已购买
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end
    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买首充礼包",
            context= "购买首充礼包",
            mailgen={subject=2, body=""},
            notify={
                flags="first_recharge1_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    THIS.gift_unlock(character)
    return 0
end

function THIS.first_recharge_award(character, args)
    local d = _M.assert_get(character)
    if not d.state then
        return STD_ERR.COMMON_AWARD_NOT_UNLOCK
    end

    local day = args.day

    if not day then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if day <= 0 or day > d.day or day > MAX_DAY then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local flag = 1 << (day -1)
    if d.award & flag ~= 0 then
        return STD_ERR.COMMON_AWARD_RECEIVED -- 奖励已领取
    end

    local conf = get_conf(character)
    local award_conf
    if day == 1 then
        award_conf = conf.parameter1
    elseif day == 2 then
        award_conf = conf.parameter2
    elseif day == 3 then
        award_conf = conf.parameter3
    else
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    local award_list = {}
    for i = 1, #award_conf, 2 do
        local item_id = award_conf[i]
        local item_num = award_conf[i+1]
        if item_num and item_num > 0 then
            table.insert(award_list, {id = item_id, num = item_num})
        else
            break
        end
    end
    if award_list and next(award_list) then
        local desc = {
            module="first_recharge_award",
            brief="首充登录奖励",
            context= string.format("首充登录奖励, moduleid:%d, day:%d", module_id, day),
            mailgen={subject=2, body=""},
            notify={
                flags="first_recharge_award"
            },
            detail=award_list
        }
        reward(character, desc)
    end
    d.award = d.award | flag
    _M.persist(character)
    return 0, {award = d.award, moduleid = module_id}
end

function THIS.gift_unlock(character)
    local d = _M.assert_get(character)
    if d.state then
        logger.warn("gift_unlock err", module_name)
        return
    end
    d.state = true
    d.day = 1
    d.start_time = util.today()+12*HOUR_SEC
    d.award = 0
    _M.persist(character)
    character.send("first_recharge_nty", {
        moduleid = module_id, 
        state = d.state, 
        day = math.min(d.day, MAX_DAY),
        award = d.award})
end

function THIS.get_client_data(character)
    local d = _M.assert_get(character)
    return 0, {
        moduleid = module_id,
        state = d.state,
        day = math.min(d.day, MAX_DAY),
        award = d.award,
    }
end

function MODULE.first_recharge_buy(character)
    return func_ret("first_recharge_buy", character)
end

function MODULE.get_client_data(character)
    return func_ret("get_client_data", character)
end

function MODULE.first_recharge_award(character, args)
    return func_ret("first_recharge_award", character, args)
end

function REQUEST.first_recharge_data(character, args)
    if not args.moduleid then
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end
    if args.moduleid == module_id then
        return MODULE.get_client_data(character)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE2 then
        return first_recharge2.get_client_data(character)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE3 then
        return first_recharge3.get_client_data(character)
    else
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end
end

function REQUEST.first_recharge_buy(character, args)
    if not args.moduleid then
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end
    if args.moduleid == module_id then
        return MODULE.first_recharge_buy(character)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE2 then
        return first_recharge2.first_recharge_buy(character)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE3 then
        return first_recharge3.first_recharge_buy(character)
    else
        return {errno = STD_ERR.COMMON_PARM_ERR1} -- 参数异常
    end
end

function REQUEST.first_recharge_award(character, args)
    if not args.moduleid then
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end
    if args.moduleid == module_id then
        return MODULE.first_recharge_award(character, args)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE2 then
        return first_recharge2.first_recharge_award(character, args)
    elseif args.moduleid == MODULE_ID_FIRST_RECHARGE3 then
        return first_recharge3.first_recharge_award(character, args)
    else
        return {errno = STD_ERR.COMMON_PARM_ERR} -- 参数异常
    end
end

function MODULE.check_over(character, args)
    local d = _M.assert_get(character)
    if d.state and d.day >= MAX_DAY then
        for i = 1, MAX_DAY do
            local flag = 1 << i-1
            if flag & d.award == 0 then
                return false
            end
        end
        return true
    end
    return false
end

function MODULE.get_red_point(character)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return 
    end
    local d = _M.assert_get(character) or {}
    if d.state then
        local day = math.min(d.day, MAX_DAY)
        local flag = (1<<day) -1
        if d.award < flag then
            return true, module_id
        end
    end

    local red_point = false 
    red_point = first_recharge2.get_red_point(character)
    if red_point then
        return true, module_id 
    end

    red_point = first_recharge3.get_red_point(character)
    if red_point then
        return true, module_id 
    end
end

return MODULE