-- 连续充值
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local reward = require "model.reward"
local common_fun = require "model.common_fun"
local activitytimemod

local module_name = "keep_recharge"
local module_id = MODULE_ID_KEEP_RECHARGE
local _M = schema.new(module_name, {
    day = 0,
    num = 0,
    award = 0,
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN} -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        local state = activitytimemod.get_state(character, module_id)
        if state == STATE_OPEN then
            local d = _M.assert_get(character)
            d.num = d.num + moneynum
            _M.persist(character)
        end
    end)

    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character) or {}
        if d.num > 0 then
            local state = activitytimemod.get_state(character, module_id)
            if state == STATE_OPEN then
                local conf = get_conf(character)
                if conf and conf.parameter1[1] and conf.parameter1[1] <= d.num then
                    d.day = d.day + 1
                    d.num = 0 
                end
            else
                d.num = 0
            end
            _M.persist(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
    local d = _M.assert_get(character)
    d.day = 0
    d.num = 0
    d.award = 0
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end

function THIS.keep_recharge_award(character, args)
    local day = args.day
    if not day or day < 0 then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local d = _M.assert_get(character) or {}

    local flag = 1 << day - 1
    if flag & d.award > 0 then
        return STD_ERR.COMMON_AWARD_RECEIVED -- 奖励已领取
    end

    local conf = get_conf(character)
    if not conf then
        return STD_ERR.COMMON_SYS_ERR -- 系统异常
    end
    if day > d.day then
        if day > d.day + 1 or d.num < conf.parameter1[1] then
            return STD_ERR.COMMON_AWARD_NOT_UNLOCK -- 没到领取条件
        end
    end

    local awardid = conf.parameter2[day]
    if not awardid or awardid == 0 then
        return STD_ERR.COMMON_NOT_AWARD -- 没有可领取的奖励
    end

    local errno, award_list = common_fun.get_award(awardid, "NewawardConfig_proto")
    if errno ~= 0 then
        return errno
    end

    if award_list and next(award_list) then
        local desc = {
            module="keep_recharge",
            brief="持续充值",
            context= "持续充值day:"..day,
            mailgen={subject=2, body=""},
            notify={
                flags="keep_recharge_award"
            },
            detail=award_list
        }
        reward(character, desc)
    end

    d.award = d.award | flag
    _M.persist(character)
    return 0, {award = d.award}
end

function THIS.keep_recharge_data(character, args)
    local d = _M.assert_get(character)
    return 0, {day = d.day, num = d.num, award = d.award}
end

function REQUEST.keep_recharge_data(character, args)
    return func_ret("keep_recharge_data", character, args)
end

function REQUEST.keep_recharge_award(character, args)
    return func_ret("keep_recharge_award", character, args)
end


function MODULE.get_red_point(character)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return 
    end

    local d = _M.assert_get(character) or {}
    local conf = get_conf(character)
    if not conf then
        return 
    end
    local max_day = #conf.parameter2
    local day = d.day
    if conf.parameter1[1] <= d.num then
        day = day + 1
    end
    day = math.min(day, max_day)
    local flag = 1 << day - 1
    if d.award < flag then
        return true, module_id
    end
end

return MODULE