-- 月卡
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local rechargemod = require "model.rechargemod"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local activitytimemod

local module_name = "supreme_card"
local module_id = MODULE_ID_SUPREME_CARD
local _M = schema.new(module_name, {
    state = false,  -- 是否购买
    start_time = 0, -- 购买的时间
    award = false,  -- 是否领取每日奖励
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN}
 -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.card_unlock(character)
        end
    end)

    character.monitor("daily_refresh",function()
        local d = _M.assert_get(character)
        if d.state then
            local change = false
            -- 检查是否是否到时间
            local conf = get_conf(character)
            if conf and conf.parameter1 then
                if conf.parameter1[1] > 0 then
                    local opentime = util.today(d.start_time)
                    local now = os.time()
                    if now - opentime > conf.parameter1[1] * DAY_SEC then
                        d.state = false
                        change = true
                    end
                end
            end
            if d.award then
                d.award = false
                change = true
            end
            if change then
                _M.assert_get(character)
            end            
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytimemod = require "model.activitytimemod"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end

function MODULE.get_client_data(character)
    local d = _M.assert_get(character)
    return {
        moduleid = module_id,
        state = d.state,
        start_time = util.today(d.start_time),
        award = d.award,
    }
end

function MODULE.card_buy(character)
    local d = _M.assert_get(character)
    local conf = get_conf(character)
    if not conf.gift[1] then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    
    local gift_conf = asset.GiftConfig_proto[conf.gift[1]]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    if d.state then
        return 2 -- 已购买
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end
    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买至尊卡",
            context= "购买至尊卡",
            mailgen={subject=2, body=""},
            notify={
                flags="card_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    THIS.card_unlock(character)
    return 0
end

function MODULE.card_award(character)
    local d = _M.assert_get(character)
    if d.state and not d.award then
        local conf = get_conf(character)
        if not conf or not conf.parameter2 or conf.parameter2[1] == 0 then
            return 1 -- 参数异常
        end
        local award_list = {}
        for i = 1, #conf.parameter2, 2 do
            local item_id = conf.parameter2[i]
            local item_num = conf.parameter2[i+1]
            if item_num and item_num > 0 then
                table.insert(award_list, {id = item_id, num = item_num})
            else
                break
            end
        end
        if award_list and next(award_list) then
            local desc = {
                module="card_award",
                brief="特权卡每日奖励",
                context= string.format("特权卡每日奖励, moduleid:%d", module_id),
                mailgen={subject=2, body=""},
                notify={
                    flags="privilege_card_award"
                },
                detail=award_list
            }
            reward(character, desc)
        end
        d.award = true
        _M.persist(character)
        return 0, {moduleid = module_id, award = d.award}
    else
        return 1 -- 参数异常
    end
end

function THIS.card_unlock(character)
    local d = _M.assert_get(character)
    if d.state then
        logger.warn("card_unlock err", module_name)
    end
    d.state = true
    d.start_time = util.today()+12*HOUR_SEC
    d.award = false
    character.send("privilege_card_unlock_nty", {data = {moduleid = module_id, state = d.state, start_time = util.today(d.start_time), award = d.award}})
    _M.persist(character)
end

function MODULE.card_unlock(character)
    local d = _M.assert_get(character)
    return d.state
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}
    if d.state then
        if not d.award then
            return true, module_id
        end
    end
end

return MODULE