-- 开服宝藏
local skynet = require "skynet"
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"
local payment = require "model.payment"
local reward = require "model.reward"
local util = require "util"
local common_fun = require "model.common_fun"
local module_fun 
local activitytimemod
local activitytime 

local module_name = "treasure"
local module_id = MODULE_ID_TREASURE
local _M = schema.new(module_name, {
   buy = 0, -- 购买次数
   sign = 0, -- 签到天数
   list = {}, 
   treasure = false,
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local state = activitytimemod.get_state(character, module_id)
    if state == STATE_CLOSE then
        return {errno = STD_ERR.COMMON_ACT_NOT_OPEN}
 -- 活动未开启
    end
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

local function get_conf(character)
    return activitytimemod.get_conf(character, module_id)
end

local function init_data(character)
    local d = _M.assert_get(character)
    d.buy = 0; -- 购买次数
    d.sign = 0; -- 签到天数
    d.list = {}; 
    d.treasure = false;
    _M.persist(character)
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    activitytime = skynet.localname(".activitytime")
    activitytimemod = require "model.activitytimemod"
    module_fun = require "model.module_fun"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.open(character, ainfo)
    init_data(character)
end

function MODULE.close(character, ainfo)
end

function MODULE.continue_open(character, ainfo)
end

function THIS.treasure_buy(character)
    local d = _M.assert_get(character) or {}

    local conf = asset.DataConfig_proto[71]
    if not conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    if d.buy >= conf.data3 then
        return STD_ERR.COMMON_BUY_LIMIT -- 购买次数不足
    end

    local item_id = conf.data4
    local item_num = conf.data5[math.min(d.buy+1, #conf.data5)]
    if not item_id or not item_num or item_num <= 0 then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local pay_list = {{
        id = item_id,
        num = item_num
    }}


    item_id = conf.data6[1]
    item_num = conf.data6[2]
    if not item_id or not item_num or item_num <= 0 then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local award_list = {{
        id = item_id,
        num = item_num,
    }}
    local desc = {
        module="treasure_buy",
        brief="购买开服宝藏抽奖次数",
        context= string.format("购买开服宝藏抽奖次数, 第%d次", d.buy+1),
        mailgen={subject=2, body=""},
        notify={
            flags="treasure_buy"
        },
        detail=award_list
    }

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买开服宝藏抽奖次数",
            context= "购买开服宝藏抽奖次数",
            mailgen={subject=2, body=""},
            notify={
                flags="treasure_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end

    reward(character, desc)
    d.buy = d.buy + 1
    _M.persist(character)
    return 0, {buy = d.buy}
end

function THIS.treasure_sign(character, args)
    local d = _M.assert_get(character) or {}
    local day = skynet.call(activitytime, "lua", "open_day")
    if d.sign >= day then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local conf = asset.DataConfig_proto[72]
    if not conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    local temp_sign = d.sign
    local award_list = {}
    for i, v in ipairs(conf.data5) do
        if v > d.sign and v <= day then
            temp_sign = math.max(temp_sign, v)
            local item_id = conf.data6[i*2-1]
            local item_num = conf.data6[i*2]
            if not item_id or not item_num then
                return STD_ERR.COMMON_CONF_ERR -- 配置异常
            end
            table.insert(award_list, {
                id = conf.data6[i*2-1],
                num = conf.data6[i*2]
            })
        end
    end

    local desc = {
        module="treasure_sign",
        brief="开服宝藏签到",
        context= string.format("开服宝藏签到, %d=>%d", d.sign, temp_sign),
        mailgen={subject=2, body=""},
        notify={
            flags="treasure_sign"
        },
        detail=award_list
    }
    d.sign = temp_sign
    reward(character, desc)
    _M.persist(character)
    return 0, {sign = d.sign}
end

function THIS.treasure_draw(character, args)
    local d = _M.assert_get(character) or {}
    local conf = get_conf(character)
    if d.treasure then
        return 1 -- 结束
    end
    local pay_list = {}
    local data_conf = asset.DataConfig_proto[71]
    if not data_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end
    table.insert(pay_list, {
        id = data_conf.data1,
        num = data_conf.data2,
    })

    if not next(pay_list) then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local receipt = {
        module=module_name,
        brief="开服宝藏抽奖",
        context= "开服宝藏抽奖",
        mailgen={subject=2, body=""},
        notify={
            flags="treasure_draw"
        },
        detail=pay_list
    }
    local errno = module_fun.paymeny_check(character, pay_list)
    if errno ~= 0 then
        return errno
    end
    local award_list = {}
    local pos = 0
    if #conf.parameter1 > #d.list then
        local temp = {}
        for _, v in ipairs(d.list) do
            temp[v] = 1
        end
        local draw_list = {}
        for i = 1, #conf.parameter1 do
            if not temp[i] then
                table.insert(draw_list, i)
            end
        end
        local size = #draw_list
        if #draw_list <= 0 then
            return STD_ERR.COMMON_SYS_ERR -- 系统异常
        end
        local randnum = size == 1 and 1 or util.rand( 1, #draw_list)
        pos = draw_list[randnum]
        local item_id = conf.parameter1[pos]
        local item_num = conf.parameter2[pos]
        if not item_id or not item_num then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        table.insert(award_list, {id = item_id, num = item_num})
    else
        local item_id = conf.parameter3[1]
        local item_num = conf.parameter3[2]
        if not item_id or not item_num then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        table.insert(award_list, {id = item_id, num = item_num})
    end
    local desc = {
        module="treasure_sign",
        brief="开服宝藏抽奖",
        context= "开服宝藏抽奖, pos:"..pos,
        mailgen={subject=2, body=""},
        notify={
            flags="treasure_draw"
        },
        detail=award_list
    }
    payment(character, receipt, true)
    reward(character, desc)
    if pos ~=0 then
        table.insert(d.list, pos)
        table.sort(d.list)
    else
        d.treasure = true
    end
    _M.persist(character)
    return 0, {pos = pos}
end

function THIS.treasure_get_data(character, args)
    local d = _M.assert_get(character) or {}
    local list = common_fun.scopy(d.list or {})
    if d.treasure then
        table.insert(list, 0)
    end
    return 0, {
        buy = d.buy,
        sign = d.sign, 
        list = list,
    }
end


function REQUEST.treasure_buy(character, args)
    return func_ret("treasure_buy", character, args)
end

function REQUEST.treasure_sign(character, args)
    return func_ret("treasure_sign", character, args)
end

function REQUEST.treasure_draw(character, args)
    return func_ret("treasure_draw", character, args)
end

function REQUEST.treasure_get_data(character, args)
    return func_ret("treasure_get_data", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}
    local state = activitytimemod.get_state(character, module_id)
    if state ~= STATE_OPEN then
        return
    end
    local day = skynet.call(activitytime, "lua", "open_day")
    local conf = asset.DataConfig_proto[72]
    if not conf then
        return
    end
    for i, v in ipairs(conf.data5) do
        if v > d.sign and v <= day then
            return true, module_id
        end
    end
end

return MODULE