-- 祭坛
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local common_fun = require "model.common_fun"
local asset = require "model.asset"
local payment = require "model.payment"
local hero
local equip
local manual

local HERO_EXCHANGE = 1
local EQUIP_EXCHANGE = 2

-- 请和manual中保持一致
local EQUIP_FREE = 2
local HERO_FREE = 3 

local MAX_ALTER_RANK_NUM = 5

local module_name = "altar"
local _M = schema.new(module_name, {
    buy_num = 0,
    list = {
        --[[
            sid = 英雄唯一id
            time = 重置时间戳
            level = 上阵时等级
            id = 上阵时英雄id
        ]]
    }
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}
local ALTAR = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
    d.buy_num = d.buy_num or 0
    d.list = d.list or {}
    local num = d.buy_num + asset.DataConfig_proto[34].data2
    local size = #d.list
    if size < num then
        for i = size + 1, num do
            d.list[i] = {sid = "", time = 0}
        end
    elseif size > num then
        for i = num + 1, size do
            d.list[i] = nil
        end
    else
        return 
    end
    _M.persist(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    local ctx = _M.assert_runtime(character)
    character.monitor("hero_upgrade_info", function(_, hero_data)
        local pre_info = ALTAR.altar_info(character)
        ALTAR.altar_update(character, ctx.list, ALTAR.altar_data(hero_data))
        local cur_info = ALTAR.altar_info(character)
        if pre_info.level ~= cur_info.level or pre_info.quality ~= cur_info.quality then
            ALTAR.sned_change(character, cur_info)
        end
    end)

    character.monitor("hero_upstart_info", function(_, hero_data)
        local pre_info = ALTAR.altar_info(character)
        ALTAR.altar_update(character, ctx.list, ALTAR.altar_data(hero_data))
        local cur_info = ALTAR.altar_info(character)
        if pre_info.level ~= cur_info.level or pre_info.quality ~= cur_info.quality then
            ALTAR.sned_change(character, cur_info)
        end
    end)

    character.monitor("hero_reset_info", function(_, hero_data)
        local pre_info = ALTAR.altar_info(character)
        ALTAR.altar_update(character, ctx.list, ALTAR.altar_data(hero_data))
        local cur_info = ALTAR.altar_info(character)
        if pre_info.level ~= cur_info.level or pre_info.quality ~= cur_info.quality then
            ALTAR.sned_change(character, cur_info)
        end
    end)

    character.monitor("hero_del", function(_, del_list)
        local pre_info = ALTAR.altar_info(character)
        for _, sid in ipairs(del_list) do
            ALTAR.altar_del(character, ctx.list, sid)
        end
        local cur_info = ALTAR.altar_info(character)
        if pre_info.level ~= cur_info.level or pre_info.quality ~= cur_info.quality then
            ALTAR.sned_change(character, cur_info)
        end
    end)

end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    hero = require "model.hero"
    equip = require "model.equip"
    manual = require "model.manual"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    local ctx = _M.assert_runtime(character)
    ALTAR.altar_init(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
    logger.test("%s:ctx, %s", module_name, stringify(ctx or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
end

function MODULE.get_altar_info(character)
    return ALTAR.altar_info(character)
end

function MODULE.get_altar_battle_info(character)
    local d = _M.assert_get(character)
    local map_list = {}
    for k, v in ipairs(d.list or {}) do
        if v.sid and v.sid ~= "" then
            map_list[v.sid] = 1
        end
    end
    return map_list
end

function MODULE.altar_change_hero_info(altar_info, hero_data)
    local temp = common_fun.scopy(hero_data)
    if temp.lv < altar_info.level then
        temp.lv = altar_info.level
    end
    local cur_quality = common_fun.get_hero_quality(temp.id)
    if cur_quality < altar_info.quality then
        temp.id = temp.id + altar_info.quality - cur_quality
    end
    return temp
end

local function compare(a, b)
    if a.level > b.level then
        return true
    end
    if a.level < b.level then
        return false
    end
    return a.quality > b.quality
end

function ALTAR.sned_change(character, altar_info)
    local d = _M.assert_get(character)
    local msg = {}
    for k, v in ipairs(d.list or {}) do
        local sid = v.sid
        if sid and sid ~= "" then
            local hero_data = hero.hero_get(character, sid)
            if hero_data then
                local temp = MODULE.altar_change_hero_info(altar_info, hero_data)
                table.insert(msg, temp)
            end
        end
    end
    if next(msg) then
        character.send("hero_change_nty", {list = msg})
    end
end

function ALTAR.altar_info(character)
    local ctx = _M.assert_runtime(character)
    local list = ctx.list or {}
    local quality = 4
    local level = 1
    if #list >= MAX_ALTER_RANK_NUM then
        level = list[MAX_ALTER_RANK_NUM].level
        local min_quality = list[MAX_ALTER_RANK_NUM].quality
        for i = 1, MAX_ALTER_RANK_NUM -1 do
            if list[i].quality < min_quality then
                min_quality = list[i].quality
            end
        end
        quality = math.max(quality, min_quality)
    end
    return {
        quality = quality,
        level = level
    }
end

function ALTAR.altar_insert(list, data)
    if not list then
        return 
    end
    local size = #list
    if size == 0 then
        list[1] = data
    else
        for i = #list, 1, -1 do
            if compare(data, list[i]) then
                list[i+1] = list[i]
                list[i] = data
            else
                list[i+1] = data
                break
            end
        end
    end
    for i = MAX_ALTER_RANK_NUM+1, #list do
        list[i] = nil
    end
end

function ALTAR.altar_srot(t)
    local size = #t
	local n = MAX_ALTER_RANK_NUM
    local f = compare
	for i = 2, size do
		local num = math.min(i, n)
		if i > n and f(t[i], t[num]) then
			t[i], t[num] = t[num], t[i]
		end
		local temp = t[num]
		local j = num -1 
		while j >= 1 and f(temp, t[j]) do
			t[j+1] = t[j]
			j = j - 1
		end
		t[j+1] = temp
	end
    for i = n + 1, size do
        t[i] = nil
    end
	return t
end

function ALTAR.altar_data(data)
    return {
        sid = data.sid,
        level = data.lv,
        quality = common_fun.get_hero_quality(data.id),
    }
end

function ALTAR.altar_find(list, sid)
    if not list then
        return
    end
    for i, v in ipairs(list or {}) do
        if v.sid == sid then
            return i
        end
    end
end

function ALTAR.altar_init(character)
    local ctx = _M.assert_runtime(character)
    ctx.list = ctx.list or {}
    local hero_list = hero.hero_get_list(character) or {}
    local battle_list = MODULE.get_altar_battle_info(character)
    for sid, v in pairs(hero_list) do
        if not battle_list[sid ] then
            local data = ALTAR.altar_data(v)
            local index = ALTAR.altar_find(ctx.list, data.sid)
            if not index then
                -- if data.level > 1 or data.quality > STAR_COLOR_PURPLE then
                    ALTAR.altar_insert(ctx.list,data)
                -- end
            end
        end
    end
end

function ALTAR.altar_update(character, list, data)
    if not list then
        return
    end
    local index = ALTAR.altar_find(list, data.sid)
    if index then
        if compare(data, list[index]) then
            list[index] = data
            ALTAR.altar_srot(list)
        else
            table.remove(list, index)
            ALTAR.altar_init(character)
        end
    else
        -- if data.level <= 1 and data.quality <= STAR_COLOR_PURPLE then
        --     return
        -- end
        table.insert(list, data)
        ALTAR.altar_srot(list)
    end
end

function ALTAR.altar_del(character, list, sid)
    if not list then
        return
    end
    local index = ALTAR.altar_find(list, sid)
    if index then
        table.remove(list, index)
        ALTAR.altar_init(character)
    end
end

function THIS.altar_data(character, args)
    local ctx = _M.assert_runtime(character)
    ctx.exchange = nil
    local d = _M.assert_get(character)
    local base_list = {}
    for _, data in ipairs(ctx.list or {}) do
        table.insert(base_list, data.sid)
    end
    local equip_free = manual.manual_privilege_times(character, EQUIP_FREE)
    local hero_free = manual.manual_privilege_times(character, HERO_FREE)
    return 0, {base_list = base_list, list = d.list, buy_num = d.buy_num, equip_free = equip_free, hero_free = hero_free}
end

function THIS.altar_battle(character, args)
    local sid = args.sid or ""
    local pos = args.pos
    if not pos then
        return 1 -- 参数异常
    end
    local d = _M.assert_get(character)
    local ctx = _M.assert_runtime(character)
    local data = d.list[pos]
    if not data then
        return 2 -- 参数异常
    end
    -- 下阵
    if sid == "" then
        if data.sid == "" then
            return 3 -- 参数异常
        end
            local hero_data = hero.hero_get(character, data.sid)
            data.sid = ""
            data.time = os.time() + asset.DataConfig_proto[34].data5[1]*60
            if hero_data then
                character.send("hero_change_nty", {list = {hero_data}})
                ALTAR.altar_update(character, ctx.list, ALTAR.altar_data(hero_data))
            end
    else
        if data.sid ~= "" then
            return 4 -- 参数异常
        end
        local now = os.time()
        if now < data.time then
            return 6 -- cd中
        end

        for _, v in pairs(d.list) do
            if v.sid == sid then
                return 7 -- 已经上阵     
            end
        end

        for _, v in ipairs(ctx.list or {}) do
            if sid == v.sid then
                return  8 -- 无法上阵祭坛中的英雄
            end
        end

        local hero_data = hero.hero_get(character, sid)
        if not hero_data then
            return 5 -- 参数异常
        end
        local altar_info = ALTAR.altar_info(character)
        local change_info = MODULE.altar_change_hero_info(altar_info, hero_data)
        character.send("hero_change_nty", {list = {change_info}})
        data.sid = sid
        data.time = 0
        data.level = assert(hero_data.lv)
        data.id = assert(hero_data.id)
    end
    _M.persist(character)
    return 0, {pos = args.pos, data = data}
end

function THIS.altar_buy_battle_space(character, args)
    local d = _M.assert_get(character)
    local data_conf = asset.DataConfig_proto[34]
    if d.buy_num >= data_conf.data3 then
        return 1 -- 无法再购买上阵位
    end
    local pay_list = {{id = CURRENCY_ID_DIA, num = data_conf.data4}}
    local receipt = {
        module="altar",
        brief= "购买祭坛上阵位",
        context= "购买祭坛上阵位",
        mailgen={subject=2, body=""},
        notify={
            flags="altar_buy_battle_space"
        },
        detail=pay_list
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return errno
    end

    d.buy_num = d.buy_num + 1
    local index = #d.list  + 1
    d.list[index] = {sid = "", time = 0}
    _M.persist(character)
    return 0, {pos = index, data = d.list[index]}
end

function THIS.altar_reset_cd(character, args)
    local pos = args.pos 
    if not pos then
        return 1 -- 参数异常
    end
    local d = _M.assert_get(character)
    local data = d.list[pos]
    if not data then
        return 2 -- 参数异常
    end

    local cd_time = data.time or 0
    local now = os.time()
    if now > cd_time then
        return 3 -- 已重置 
    end

    local times = math.ceil((cd_time - now)/3600)
    local data_conf = asset.DataConfig_proto[34]
    local pay_list = {{id = CURRENCY_ID_DIA, num = data_conf.data6[1]*times}}
    local receipt = {
        module="altar",
        brief= "重置祭坛cd时间",
        context= "重置祭坛cd时间",
        mailgen={subject=2, body=""},
        notify={
            flags="altar_reset_cd"
        },
        detail=pay_list
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return errno
    end
    data.time = 0
    _M.persist(character)
    return 0, {pos = args.pos, data = data}
end

function THIS.altar_hero_exchenge(character, sid)
    local hero_data = hero.hero_get(character, sid)
    if not hero_data then
        return 2 -- 参数异常
    end

    local quality = common_fun.get_hero_quality(hero_data.id)
    local data_conf = asset.DataConfig_proto[35]
    local item_id = data_conf.data1
    local item_num
    local award_id
    if quality == STAR_COLOR_PURPLE then
        item_num = data_conf.data2
        award_id = data_conf.data5[1]
    elseif quality == STAR_COLOR_ORANGE then
        item_num = data_conf.data3
        award_id = data_conf.data5[2]
    else
        return 3 -- 不是可以置换的星级
    end

    if not item_num then
        return 4 -- 配置异常
    end

    local errno, award_list = common_fun.get_award(award_id)
    if errno ~= 0 then
        return errno
    end
    local _, list = next(award_list or {})
    if not list or not list.id  then
        return 4 -- 配置异常
    end

    local hero_conf_list = assert(asset.RoleConfig_proto, "RoleConfig_proto")
    if not hero_conf_list[list.id] then
        return 4 -- 配置异常
    end
    local hero_free = manual.manual_privilege_times(character, HERO_FREE)
    if hero_free > 0 then
        manual.manual_privilege_use(character, HERO_FREE)
    else
        local pay_list = {{id = item_id, num = item_num}}
        local receipt = {
            module="altar",
            brief= "英雄置换",
            context= "英雄置换:"..quality,
            mailgen={subject=2, body=""},
            notify={
                flags="altar_exchange"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    local ctx = _M.assert_runtime(character)
    ctx.exchange = {
        type = HERO_EXCHANGE,
        sid = hero_data.sid,
        level = hero_data.lv,
        id = hero_data.id,
        change = list.id
    }
    return 0, {type = HERO_EXCHANGE, sid = sid, id = list.id}
end

function THIS.altar_equip_exchenge(character, sid)
    local equip_data = equip.equip_get(character, sid)
    if not equip_data then
        return 2 -- 参数异常
    end

    local quality = common_fun.get_equip_quality(equip_data.id)
    local data_conf = asset.DataConfig_proto[36]
    local item_id = data_conf.data1
    local item_num
    local award_id

    
    local equip_conf_list = assert(asset.ArmorConfig_proto, "ArmorConfig_proto")
    local equip_conf = equip_conf_list[equip_data.id]
    if not equip_conf then
        return 3 -- 系统异常
    end

    local award_conf = asset.DataConfig_proto[38+equip_conf.type]
    if not award_conf then
        return 4 -- 配置异常
    end

    if quality == STAR_COLOR_PURPLE then
        item_num = data_conf.data2
        award_id = award_conf.data1
    elseif quality == STAR_COLOR_ORANGE then
        item_num = data_conf.data3
        award_id = award_conf.data2
    else
        return 3 -- 不是可以置换的星级
    end

    if not item_num then
        return 4 -- 配置异常
    end

    local errno, award_list = common_fun.get_award(award_id)
    if errno ~= 0 then
        return errno
    end
    local _, list = next(award_list or {})
    if not list or not list.id  then
        return 4 -- 配置异常
    end
    if not equip_conf_list[list.id] then
        return 4 -- 配置异常
    end

    local equip_free = manual.manual_privilege_times(character, EQUIP_FREE)
    if equip_free > 0 then
        manual.manual_privilege_use(character, EQUIP_FREE)
    else
        local pay_list = {{id = item_id, num = item_num}}
        local receipt = {
            module="altar",
            brief= "装备置换",
            context= "装备置换:"..quality,
            mailgen={subject=2, body=""},
            notify={
                flags="altar_exchange"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    local ctx = _M.assert_runtime(character)
    ctx.exchange = {
        type = EQUIP_EXCHANGE,
        sid = equip_data.sid,
        level = equip_data.lv,
        id = equip_data.id,
        change = list.id
    }
    return 0, {type = EQUIP_EXCHANGE, sid = sid, id = list.id}
end

function THIS.altar_exchange(character, args)
    local etype = args.type
    local sid = args.sid
    if not etype or not sid then
        return 1 -- 参数异常
    end
    local ctx = _M.assert_runtime(character)
    ctx.exchange = nil
    if etype == HERO_EXCHANGE then
        return THIS.altar_hero_exchenge(character, sid)
    elseif etype == EQUIP_EXCHANGE then
        return THIS.altar_equip_exchenge(character, sid)
    end
    return 0
end

function THIS.altar_exchange_confirm(character, args)
    local etype = args.type
    local sid = args.sid
    local ctx = _M.assert_runtime(character)
    if not etype or not sid or not ctx.exchange then
        return 1 -- 参数异常
    end
    if etype ~= ctx.exchange.type then
        return 2 -- 参数异常
    end
    if sid ~= ctx.exchange.sid then
        return 2 -- 参数异常
    end

    if not args.change then
        ctx.exchange = nil
        return 0
    else
        if etype == HERO_EXCHANGE then
            local hero_data = hero.hero_get(character, sid)
            if hero_data.id ~= ctx.exchange.id or hero_data.lv ~= ctx.exchange.level then
                return 11 -- 已过有效期
            end
            hero_data.id = ctx.exchange.change
            hero.hero_save(character)
            character.send("hero_change_nty", {list = {hero_data}})
        elseif etype == EQUIP_EXCHANGE then
            local euiip_data = equip.equip_get(character, sid)
            if euiip_data.id ~= ctx.exchange.id or euiip_data.lv ~= ctx.exchange.level then
                return 11 -- 已过有效期
            end
            euiip_data.id = ctx.exchange.change
            equip.equip_save(character)
            character.send("equip_change_nty", {list = {euiip_data}})
        else
            return 4 -- 系统异常
        end
    end
    return 0
end 

function REQUEST.altar_data(character, args)
    return func_ret("altar_data", character, args)
end

function REQUEST.altar_battle(character, args)
    return func_ret("altar_battle", character, args)
end


function REQUEST.altar_reset_cd(character, args)
    return func_ret("altar_reset_cd", character, args)
end

function REQUEST.altar_buy_battle_space(character, args)
    return func_ret("altar_buy_battle_space", character, args)
end

function REQUEST.altar_exchange(character, args)
    return func_ret("altar_exchange", character, args)
end

function REQUEST.altar_exchange_confirm(character, args)
    return func_ret("altar_exchange_confirm", character, args)
end




return MODULE