local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"
local common_fun = require "model.common_fun"
local payment = require "model.payment"
local reward = require "model.reward"

local skill_card

local MAX_CHANGE = 20000000000
local module_name = "debris"
local _M = schema.new(module_name, {
    list = {
        -- [id] = {id = id, num = num}
    },
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)

end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    skill_card = skill_card or require "model.skill_card"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    -- logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function MODULE.add_debris(character, id, num, collect, dispose, addition)
    local d = _M.assert_get(character)
    if not id or not num then
        dispose(nil, id, num)
    end

    local conf_list = assert(asset.CardConsumeConfig_proto, "CardConsumeConfig_proto")
    local conf = conf_list[id]
    -- 一次超过2000亿是什么数值
    if not conf and dispose or num <=0 or num > MAX_CHANGE then
        dispose(nil, id, num)
        return 
    end

    d.list[id] = d.list[id] or {id = id, num = 0}
    d.list[id].num = d.list[id].num + math.floor(num) -- 向下取下整避免意外
    addition(GOODS_DEBRIS, id, num)
    collect(GOODS_DEBRIS, d.list[id])
    _M.persist(character)
    return true, num, d.list[id].num
end

function MODULE.pay_debris(character, id, num)
    local d = _M.assert_get(character)
    if num < 0 then
        assert(false)
    end
    local cur_num = 0
    if d.list and d.list[id] and d.list[id].num then
        cur_num = d.list[id].num
    else
        assert(false)
    end
    if cur_num < num then
        assert(false)
    end
    d.list[id].num = d.list[id].num - math.floor(num)
    _M.persist(character)
    return  d.list[id].num
end

function MODULE.check_debris(character, id, num)
    local d = _M.assert_get(character)
    -- 数量至少为1
    if num < 0 then
        return STD_ERR.PAYMENT_NUM_ERR
    end

    if not d.list or not d.list[id] then
        local card_conf_list = assert(asset.CardConsumeConfig_proto, "CardConsumeConfig_proto")
        if not card_conf_list[id] then
            return STD_ERR.PAYMENT_UNKNOWN_ITEM
        end
        return STD_ERR.PAYMENT_NOT_ENOUGH
    end

    if not d.list[id].num then
        return STD_ERR.COMMON_SYS_ERR
    end

    if d.list[id].num < num then
        return STD_ERR.PAYMENT_NOT_ENOUGH
    end

    return 0
end

function MODULE.get_debris(character, id)
    local d = _M.assert_get(character)
    if not d.list or not d.list[id] then
        return 0
    end
    return d.list[id].num
end

function MODULE.get_data(character)
    local d = _M.assert_get(character)
    return d.list
end

function THIS.debris_get_data(character)
    return 0, {list = common_fun.tqueue(MODULE.get_data(character))}
end

function THIS.debris_merge(character, args)
    local d = _M.assert_get(character)
    local id = args.id
    if not id then
        return 1  -- 参数异常
    end
    local card_conf_list = assert(asset.CardConsumeConfig_proto, "CardConsumeConfig_proto")
    local card_conf = card_conf_list[id]
    if not card_conf then
        return 1 -- 参数异常
    end

    -- 检查是否解锁技能
    if skill_card.skill_card_check_skill(character, card_conf.card) then
        return 2 -- 技能已激活
    end
    
    local receipt = {
        module="debris",
        brief="碎片合成",
        context="碎片合成:"..id,
        notify={
            flags="debris_merge"
        },
        detail={{id = id, num = card_conf.num}}
    }
    local errno = payment(character, receipt)
    if errno ~= 0 then
        return errno
    end

    local desc = {
        module="debris",
        brief="碎片合成",
        context= "碎片合成:"..id,
        mailgen={subject=2, body=""},
        notify={
            flags="debris_merge"
        },
        detail={{id = card_conf.card, num  = 1}}
    }   
    reward(character, desc)
    return 0
end

function REQUEST.debris_get_data(character, args)
    return func_ret("debris_get_data", character, args)
end

function REQUEST.debris_merge(character, args)
    return func_ret("debris_merge", character, args)
end

return MODULE