local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local util = require "util"
local asset = require "model.asset"
local common_fun = require "model.common_fun"
local module_fun = require "model.module_fun"
local payment = require "model.payment"
local reward = require "model.reward"
local skynet = require "skynet"
local equip
local hero 
local ser_mail = skynet.localname(".mailbox")

local HERO_DRAW = 1     -- 英雄高级召唤
local HERO_DRAW2 = 2    -- 英雄普通召唤
local EQUIP_DRAW = 3    -- 装备锻造

local PAY_TYPE_FREE = 1     -- 免费
local PAY_TYPE_TICKET = 2   -- 召唤券
local PAY_TYPE_CURRENCY = 3 -- 其它货币

local module_name = "draw"
local module_id = MODULE_ID_DIA_DRAW
local _M = schema.new(module_name, {
    -- hero = { 英雄召唤
    --     free = 0,   -- 已免费次数
    --     daily = 0,  -- 每日次数
    --     all = 0,    -- 总次数 
    --     list = {
    --         --[[
    --             [id] = {
    --                 score = 0, -- 积分
    --                 times = 0, -- 抽中次数
    --             }
    --         ]]
    --     }
    -- },
    -- hero2  普通英雄召唤
    -- equip
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local new_data = function()
    return {
        free = 0,
        daily = 0,
        all = 0,
        list = {},
    }    
end


local function draw(data, conf_list, num, special_list)
    local award_list = {}
    local id_list = {}
    local times = num
    while(times > 0) do
        local awardid = 0
        local awardtimes = 0
        data.all = data.all + 1
        data.daily = data.daily + 1
        -- 先随特殊库, 看下特殊库的必出是否达成
        for _, conf in ipairs(special_list or {}) do
            if conf.must > 0 then
                local id = conf.ID
                local list = data.list
                if not data.list[id] then
                    list[id] = {
                        score = 0,
                        times = 0
                    }
                end
                local cur_score = data.all - list[id].score
                if cur_score >= conf.must then
                    list[id].score = data.all
                    list[id].times = list[id].times + 1
                    awardid = conf.points
                    awardtimes = list[id].times
                    break
                end
            end
        end

        if awardid == 0 then
            for _, conf in ipairs(conf_list) do
                local id = conf.ID
                local list = data.list
                if not list[id] then
                    list[id] = {
                        score = 0,
                        times = 0
                    }
                end
                local cur_score = data.all - list[id].score
                if awardid == 0 and conf.must > 0 and cur_score >= conf.must then
                    list[id].score = data.all
                    list[id].times = list[id].times + 1
                    awardid = conf.points
                    awardtimes = list[id].times
                    break
                end

                if awardid == 0 and cur_score >= conf.enter then
                    local score = (cur_score - conf.enter)
                    local all = conf.luckya*score^conf.luckyn+conf.luckyb*score+conf.chance
                    local randnum = util.rand(1, MAXRAND)
                    if all >= randnum then
                        list[id].score = data.all
                        list[id].times = list[id].times + 1
                        awardid = conf.points
                        awardtimes = list[id].times
                        break
                    end
                end
            end
        end

        if awardid == 0 then
            local conf = conf_list[#conf_list]
            local id = conf.ID
            local list = data.list
            list[id].score = 0
            list[id].times = list[id].times + 1
            awardid = conf_list[#conf_list].points
            awardtimes = list[id].times
        end
        id_list[awardid] = (id_list[awardid] or 0) + 1
        local ok, temp = common_fun.get_award(awardid, "CallawardConfig_proto", awardtimes)
        if ok then
            award_list = common_fun.merge2list(award_list, temp)
        else
            break
        end
        times = times - 1
    end
    if times ~= 0 then
        return {}
    end
    return award_list, id_list
end

local function get_clent_data(data, special_list)
    local ret = {}
    ret.free = data.free
    ret.daily = data.daily
    ret.list = {}
    for _, conf in ipairs(special_list) do
        local id = conf.ID
        local must = conf.must or 0
        if must == 0 then
            break
        end
        local use_score = data.list[id] and  data.list[id].score or 0
        local score = data.all - use_score
        local next_times = math.max(1, must - score)
        table.insert(ret.list, next_times)
    end
    return ret
end

local func_map = {}
func_map[HERO_DRAW] = function(character, ptype, num)
    local d = _M.assert_get(character)
    local data = d.hero 
    local pay_list = {}
    if ptype == PAY_TYPE_FREE then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    elseif ptype == PAY_TYPE_TICKET then
        local item_id = assert(asset.DataConfig_proto[15], "DataConfig_proto:15").data1
        table.insert(pay_list, {id = item_id, num = num})
        local errno = module_fun.paymeny_check(character, pay_list)
        if errno ~= 0 then
            return errno
        end
    elseif ptype == PAY_TYPE_CURRENCY then
        local item_num = assert(asset.DataConfig_proto[15], "DataConfig_proto:15").data4
        table.insert(pay_list, {id = CURRENCY_ID_DIA, num = item_num})
        local errno = module_fun.paymeny_check(character, pay_list)
        if errno ~= 0 then
            return errno
        end
    end
    if data.daily + num > asset.DataConfig_proto[15].data3 then
        return STD_ERR.DRAW_DAILY_TIMES_LIMIT -- 超过次数限制
    end


    local data_conf = asset.DataConfig_proto[20]
    local special_list = {}
    local special_conf = asset.HeroescallConfig_proto[data_conf.data2]
    if special_conf then
        table.insert(special_list, special_conf)
    end

    local conf_list = {}
    for _, v in pairs(asset.HeroescallConfig_proto) do
        if v.type == HERO_DRAW then
            table.insert(conf_list, v)
        end
    end

    table.sort(conf_list, function(a, b) 
        if a.priority > b.priority then
            return true
        end
        return false
    end)

    local temp = common_fun.scopy(data)
    local award_list = draw(temp, conf_list, num, special_list)
    if not next(award_list) then
        return STD_ERR.COMMON_SYS_ERR -- 系统异常
    end
    -- 发奖励
    local desc = {
        module="draw",
        brief="高级英雄抽奖",
        mailgen={subject=2, body=""},
        context=string.format("高级英雄抽奖, num:%d", num),
        notify={
            flags="draw"
        },
        detail=award_list
    }

    if next(pay_list) then
        local receipt = {
            module="draw",
            brief="高级英雄抽奖",
            context=string.format("高级英雄抽奖, num:%d", num),
            notify={
                flags="draw"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~=0 then 
            return errno
        end
    end

    reward(character, desc)
    data = temp
    if ptype == PAY_TYPE_FREE then
        data.free = data.free + num
    end
    d.hero = data
    _M.persist(character)

    local ret = get_clent_data(d.hero, special_list)
    ret.award_list = award_list
    ret.draw_type = HERO_DRAW
    character.dispatch("draw_times", num, HERO_DRAW)
    return 0, ret
end

func_map[HERO_DRAW2] = function(character, ptype, num)
    local d = _M.assert_get(character)
    local data = d.hero2
    -- 普通抽没有其他货币方式
    local pay_list = {}
    if ptype == PAY_TYPE_FREE then
        local free_times = assert(asset.DataConfig_proto[14], "DataConfig_proto:14").data3
        if data.free >= free_times then
            return STD_ERR.DRAW_FREE_TIMES_LIMIT -- 免费次数不足
        end
        num = 1
    elseif ptype == PAY_TYPE_TICKET then
        local item_id = assert(asset.DataConfig_proto[14], "DataConfig_proto:14").data1
        table.insert(pay_list, {id = item_id, num = num})
        local errno = module_fun.paymeny_check(character, pay_list)
        if errno ~= 0 then
            return errno
        end
    elseif ptype == PAY_TYPE_CURRENCY then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local data_conf = asset.DataConfig_proto[20]
    local special_list = {}
    local special_conf = asset.HeroescallConfig_proto[data_conf.data1]
    if special_conf then
        table.insert(special_list, special_conf)
    end

    local conf_list = {}
    for _, v in pairs(asset.HeroescallConfig_proto) do
        if v.type == HERO_DRAW2 then
            table.insert(conf_list, v)
        end
    end

    table.sort(conf_list, function(a, b) 
        if a.priority > b.priority then
            return true
        end
        return false
    end)

    local temp = common_fun.scopy(data)
    local award_list = draw(temp, conf_list, num, special_list)
    if not next(award_list) then
        return STD_ERR.COMMON_SYS_ERR -- 系统异常
    end
    -- 发奖励
    local desc = {
        module="draw",
        brief="普通英雄抽奖",
        mailgen={subject=2, body=""},
        context=string.format("普通英雄抽奖, num:%d", num),
        notify={
            flags="draw"
        },
        detail=award_list
    }

    if next(pay_list) then
        local receipt = {
            module="draw",
            brief="普通英雄抽奖",
            context=string.format("普通英雄抽奖, num:%d", num),
            notify={
                flags="draw"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~=0 then 
            return errno
        end
    end
    reward(character, desc)
    data = temp
    if ptype == PAY_TYPE_FREE then
        data.free = data.free + num
    end
    d.hero2 = data
    _M.persist(character)

    local ret = get_clent_data(d.hero2, special_list)
    ret.award_list = award_list
    ret.draw_type = HERO_DRAW2
    character.dispatch("draw_times", num, HERO_DRAW2)
    if ptype == PAY_TYPE_FREE then
        character.dispatch("watch_ad")
    end
    return 0, ret
end

func_map[EQUIP_DRAW] = function(character, ptype, num)
    local d = _M.assert_get(character)
    local data = d.equip
    local pay_list = {}
    if ptype == PAY_TYPE_FREE then
        local free_times = assert(asset.DataConfig_proto[16], "DataConfig_proto:16").data1
        if data.free >= free_times then
            return STD_ERR.DRAW_FREE_TIMES_LIMIT -- 免费次数不足
        end
        num = 1
    elseif ptype == PAY_TYPE_TICKET then
        local item_id = assert(asset.DataConfig_proto[16], "DataConfig_proto:16").data4
        table.insert(pay_list, {id = item_id, num = num})
        local errno = module_fun.paymeny_check(character, pay_list)
        if errno ~= 0 then
            return errno
        end
    elseif ptype == PAY_TYPE_CURRENCY then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if data.daily + num > asset.DataConfig_proto[16].data3 then
        return STD_ERR.DRAW_DAILY_TIMES_LIMIT -- 超过次数限制
    end

    local data_conf = asset.DataConfig_proto[20]
    local special_list = {}
    local special_conf = asset.HeroescallConfig_proto[data_conf.data3]
    if special_conf then
        table.insert(special_list, special_conf)
    end

    special_conf = asset.HeroescallConfig_proto[data_conf.data4]
    if special_conf then
        table.insert(special_list, special_conf)
    end

    local conf_list = {}
    for _, v in pairs(asset.HeroescallConfig_proto) do
        if v.type == EQUIP_DRAW then
            table.insert(conf_list, v)
        end
    end

    table.sort(conf_list, function(a, b) 
        if a.priority > b.priority then
            return true
        end
        return false
    end)

    local temp = common_fun.scopy(data)
    local award_list = draw(temp, conf_list, num, special_list)
    if not next(award_list) then
        return STD_ERR.COMMON_SYS_ERR -- 系统异常
    end
    -- 发奖励
    local desc = {
        module="draw",
        brief="装备锻造",
        context=string.format("装备锻造, num:%d", num),
        mailgen={subject=2, body=""},
        notify={
            flags="draw"
        },
        detail=award_list
    }

    if next(pay_list) then
        local receipt = {
            module="draw",
            brief="装备锻造",
            context=string.format("装备锻造, num:%d", num),
            notify={
                flags="draw"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt, true)
        if errno ~=0 then 
            return errno
        end
    end
    reward(character, desc)
    data = temp
    if ptype == PAY_TYPE_FREE then
        data.free = data.free + num
    end
    d.equip = data
    _M.persist(character)

    local ret = get_clent_data(d.equip, special_list)
    ret.award_list = award_list
    ret.draw_type = EQUIP_DRAW
    character.dispatch("draw_times", num, EQUIP_DRAW)
    if ptype == PAY_TYPE_FREE then
        character.dispatch("watch_ad")
    end
    return 0, ret
end

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    local d = _M.assert_get(character)
    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character)
        local change = false
        for _, v in pairs(d or {}) do
            if v.daily and v.daily ~= 0 then
                v.daily = 0
                change = true
            end
            if v.free and v.free ~= 0 then
                v.free = 0
                change = true
            end
        end
        if change then
            _M.persist(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    d.hero = d.hero or new_data()
    d.hero2 = d.hero2 or new_data()
    d.equip = d.equip or new_data()
    equip = require "model.equip"
    hero = require "model.hero"
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function THIS.draw(character, args)
    local dtype = args.draw_type
    local ptype = args.pay_type
    local num = args.num or 1

    if not dtype or dtype < HERO_DRAW or dtype > EQUIP_DRAW then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if not ptype or ptype ~= PAY_TYPE_FREE and ptype ~= PAY_TYPE_TICKET and ptype ~= PAY_TYPE_CURRENCY then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local bag_num = 0
    local errno = 0
    if dtype == HERO_DRAW or dtype == HERO_DRAW2 then
        bag_num = hero.hero_surplus(character)
        errno = STD_ERR.HERO_FULL
    elseif dtype == EQUIP_DRAW then
        bag_num = equip.equip_surplus(character)
        errno = STD_ERR.EQUIP_FULL
    end 

    if bag_num < num then
        local ok, mail_num = pcall(skynet.call, ser_mail, "lua", "surplus_mail_num", character.uid)
        if not ok then
            return STD_ERR.COMMON_BAG_MAIL_FULL
        end

        if mail_num < 0 then
            return errno
        end
    end

    return func_map[dtype](character, ptype, num)
end

function THIS.draw_get_data(character, args)
    local d = _M.assert_get(character)
    local ret = {
    }
    local data_conf = asset.DataConfig_proto[20]
    local special_conf = asset.HeroescallConfig_proto[data_conf.data1]
    local special_list = {}
    if special_conf then
        table.insert(special_list, special_conf)
    end
    ret.hero = get_clent_data(d.hero, special_list)

    special_list = {}
    special_conf = asset.HeroescallConfig_proto[data_conf.data2]
    if special_conf then
        table.insert(special_list, special_conf)
    end
    ret.hero2 = get_clent_data(d.hero2, special_list)

    special_list = {}
    special_conf = asset.HeroescallConfig_proto[data_conf.data3]
    if special_conf then
        table.insert(special_list, special_conf)
    end
    special_conf = asset.HeroescallConfig_proto[data_conf.data4]
    if special_conf then
        table.insert(special_list, special_conf)
    end
    ret.equip = get_clent_data(d.equip, special_list)
    return 0, ret
end

function REQUEST.draw(character, args)
    return func_ret("draw", character, args)
end

function REQUEST.draw_get_data(character, args)
    return func_ret("draw_get_data", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}

    local data = d.hero2
    local free_times = assert(asset.DataConfig_proto[14], "DataConfig_proto:14").data3
    if data.free < free_times then
        return true, module_id
    end

    data = d.equip
    free_times = assert(asset.DataConfig_proto[16], "DataConfig_proto:16").data1
    if data.free < free_times then
        return true, MODULE_ID_DRAW_EQUIP
    end
end


return MODULE