local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local util = require "util"
local asset = require "model.asset"
local reward = require "model.reward"

local module_name = "online_award"
local _M = schema.new(module_name, {
    open = true,
    time = 0;
    award = 0;
})

local start_data_id = 49
local end_data_id = 54

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end



local function get_online_time(character)
    local d = _M.assert_get(character) or {}
    if d.open then
        local ctx = _M.assert_runtime(character)
        local now = os.time()
        return d.time + math.max((now - (ctx.start or 0)), 0)
    end
    return 0
end

local function send_nty(character)
    local d = _M.assert_get(character) or {}
    local ret = {}
    if d.open then
        ret.open = true
        ret.time = get_online_time(character)
        ret.award = d.award
    else
        ret.open = false
    end
    character.send("online_award_nty", ret)
end

local function check_close(character)
    local d = _M.assert_get(character) or {}
    if d.open then
        local createtime = util.today(character.createtime)
        local now = os.time()
        if now > createtime + 7 * DAY_SEC then
            d.open = false
            _M.persist(character)
            send_nty(character)
        end
    end
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character)
        check_close(character)
        if d.open then
            local ctx = _M.assert_runtime(character)
            ctx.start = os.time()
            d.time = 0;
            d.award = 0;
            _M.persist(character)
            send_nty(character)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character) or {}
    check_close(character)
    if d.open then
        local ctx = _M.assert_runtime(character)
        ctx.start = os.time()
    end
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.trace("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)
    local d = _M.assert_get(character)
    if d.open then
        local ctx = _M.assert_runtime(character)
        local now = os.time()
        if ctx.start and now > ctx.start then
            d.time = (d.time + now-ctx.start)
            _M.persist(character)
        end
    end
end

function THIS.online_award_get_award(character, args)
    local d = _M.assert_get(character) or {}
    if not d.open then
        return STD_ERR.COMMON_ACT_NOT_OPEN    -- 活动已结束
    end

    local pos = args.pos
    if not pos then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local dataid = start_data_id + pos -1
    if dataid < start_data_id or dataid > end_data_id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    if pos <=0 or pos > 64 then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end

    local flage = 1 << (pos -1)

    if d.award & flage > 0 then
        return STD_ERR.COMMON_AWARD_RECEIVED -- 奖励已领取
    end

    local data_conf = asset.DataConfig_proto[dataid]
    if not data_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local online_time = get_online_time(character)
    if online_time <= data_conf.data1 then
        return STD_ERR.COMMON_NOT_AWARD_CONDITION -- 在线时间不足        
    end

    local award_list = {{id = data_conf.data2, num = data_conf.data3}}
    -- 发奖励
    local desc = {
        module="online_award",
        brief="在线奖励",
        mailgen={subject=2, body=""},
        context=string.format("在线奖励, pos:%d", args.pos),
        notify={
            flags="online_award_get_award"
        },
        detail=award_list
    }

    d.award = d.award | flage
    reward(character, desc)
    
    _M.persist(character)
    return 0, {award = d.award}
end

function REQUEST.online_award_get_award(character, args)
    return func_ret("online_award_get_award", character, args)
end

function REQUEST.online_award_get_data(character, args)
    send_nty(character)
    return {errno = 0}
end


return MODULE