-- 限时礼包(弹窗礼包)
local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"
local rechargemod = require "model.rechargemod"
local payment = require "model.payment"
local reward = require "model.reward"

local module_name = "pop_gift"
local module_id = MODULE_ID_POP_GIFT
local _M = schema.new(module_name, {
    list = {
        --[[
            [gift_id] = {
                id = 礼包id
                num = 购买次数
                time = 时间戳
                tnum = 已触发次数
            }   
        ]]
    }
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

local function get_client_gift_conf(id)
    local limit_conf = asset.LimitgiftConfig_proto[id]
    if not limit_conf then
        return 
    end
    local gift_id = limit_conf.gift
    local conf = asset.GiftConfig_proto[gift_id]
    if not conf then
        return 
    end
    return {
        spendtype = conf.spendtype,
        spend = conf.spend,
        num = conf.num,
        library = conf.library
    }
end

local function create_client_data_one(data)
    local list = {
        id = data.id,
        num = data.num, 
        time = data.time,
        conf = get_client_gift_conf(data.id)
    }
    return list
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    local d = _M.assert_get(character)
    local function func(type, condition)
        local list = {}
        local now = os.time()
        for _, conf in pairs(asset.LimitgiftConfig_proto) do
            if type == conf.type and (condition == conf.condition or 
                (type == 8 and math.floor(condition/100)==math.floor(conf.condition/100))) then
                local giftid = conf.ID
                d.list[giftid] = d.list[giftid] or {
                    id = giftid,
                    num = 0,
                    tnum = 0,
                    time = 0,
                }           
                local data = d.list[giftid]
                if conf.trigger >= data.tnum + 1 and now >= data.time then
                    data.num = 0
                    data.time = now + conf.duration * MIN_SEC
                    data.tnum = data.tnum + 1
                    table.insert(list, create_client_data_one(data))
                end
            end
        end
        if next(list) then
            _M.persist(character)
            for _, data in ipairs(list) do
                character.send("pop_gift_nty", {data = data})
            end
        end
    end

    character.monitor("role_level", function(_, pre, cur)
        for i = pre + 1, cur do
            func(1, i)
        end
    end)
    
    character.monitor("simple_battle_pass", function(_, id)
        func(2, id)
    end)
    
    character.monitor("elite_battle_pass", function(_, id)
        func(3, id)
    end)

    character.monitor("relic_diff_pass", function(_, diff)
        func(4, diff)
    end)

    character.monitor("hero_upstart", function(_, id, pre, cur)
        for i = pre + 1, cur do
            func(5, i)
        end
    end)

    character.monitor("equip_upstart", function(_, id, pre, cur)
        for i = pre + 1, cur do
            func(6, i)
        end
    end)

    character.monitor("skill_card_upgrade", function(_, id, pre, cur)
        for i = pre + 1, cur do
            func(7, i)
        end
    end)

    character.monitor("hero_add_num", function(_, id, num)
        func(8, id)
    end)

    character.monitor("recharge.money",function(_, cfid, orderid, moneynum, giftid, id, pos)
        if module_id == id then
            THIS.gift_buy(character, giftid)
        end
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function THIS.gift_buy(character, giftid)
    local d = _M.assert_get(character)
    for _, limit_conf in pairs(asset.LimitgiftConfig_proto) do
        if limit_conf.gift == giftid then
            if d.list[limit_conf.ID] then
                d.list[limit_conf.ID].num =  d.list[limit_conf.ID].num + 1
                _M.persist(character)
                character.send("pop_gift_change_nty", {data = create_client_data_one(d.list[limit_conf.ID])})
            end
            _M.persist(character)
            break
        end
    end
end

function THIS.pop_gift_data(character, args)
    local list = {}
    local d = _M.assert_get(character)
    local now = os.time()
    local limit_conf_list = asset.LimitgiftConfig_proto
    for _, data in pairs(d.list) do
        local limit_conf = limit_conf_list[data.id]
        if limit_conf and limit_conf.limit > data.num and data.time > now then
            table.insert(list, create_client_data_one(data))
        end
    end
    return 0, {list = list}
end

function THIS.pop_gift_buy(character, args)
    local d = _M.assert_get(character)

    local id = args.id
    if not id then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local data = d.list[id]
    if not data then
        return STD_ERR.COMMON_PARM_ERR -- 参数异常
    end
    local now = os.time()
    if now >= data.time then
        return STD_ERR.COMMON_GIFT_TIME_LIMIT -- 礼包已关闭
    end

    local limit_conf = asset.LimitgiftConfig_proto[id]
    if not limit_conf then
        return STD_ERR.COMMON_SYS_ERR -- 系统异常
    end

    if data.num >= limit_conf.limit then
        return STD_ERR.COMMON_NO_MORE_GIFT -- 礼包已售罄
    end 

    local gift_conf = asset.GiftConfig_proto[limit_conf.gift]
    if not gift_conf then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local pay_list = {}
    if gift_conf.spendtype == PAY_TYPE_FREE then
        pay_list = nil
    elseif gift_conf.spendtype == PAY_TYPE_ITEM then 
        table.insert(pay_list, {id = gift_conf.spend, num = gift_conf.num})
    elseif gift_conf.spendtype == PAY_TYPE_GIFT then
        rechargemod.get_orderid(character, gift_conf.ID)
        return 0
    end

    if pay_list then
        if not next(pay_list) then
            return STD_ERR.COMMON_CONF_ERR -- 配置异常
        end
        local receipt = {
            module=module_name,
            brief="购买弹窗礼包",
            context= "购买弹窗礼包"..id,
            mailgen={subject=2, body=""},
            notify={
                flags="pop_gift_buy"
            },
            detail=pay_list
        }
        local errno = payment(character, receipt)
        if errno ~= 0 then
            return errno
        end
    end
    THIS.gift_buy(character, id)
    return 0
end

function REQUEST.pop_gift_data(character, args)
    return func_ret("pop_gift_data", character, args)
end

function REQUEST.pop_gift_buy(character, args)
    return func_ret("pop_gift_buy", character, args)
end


return MODULE