local schema = require "model.schema"
local logger = require "logger"
local stringify = require "stringify"
local asset = require "model.asset"
local reward = require "model.reward"

local MAX_DAY = 7 -- 7天刷新一次

local module_id = MODULE_ID_SIGN_IN
local module_name = "sign_in"
local _M = schema.new(module_name, {
    open = false,
    day = 0,
    award = 0,
    round = 1,
})

local REQUEST = {}
local CMD = {}
local MODULE = {}
local THIS = {}

local function func_ret(fname, character, args)
    local f = THIS[fname]
    if not f then
        logger.error("func_ret not fname:%s !!!", fname)
        return {errno = STD_ERR.COMMON_SYS_ERR}
    end
    local errno, ret = f(character, args)
    if errno ~= 0 then
        return {errno = errno}
    end
    ret = ret or {}
    ret.errno = 0
    return ret
end

function MODULE.list_request_interests() return REQUEST end
function MODULE.list_command_interests() return CMD end

-- TODO: 解析/升级模块数据 在这里把数据初始化好
function MODULE.parse(character)
    local d = _M.load(character)
end

-- TODO: 侦听事件
function MODULE.monitor(character)
    character.monitor("daily_refresh", function()
        local d = _M.assert_get(character)
        if d.open then
            if d.day >= MAX_DAY then
                THIS.data_refresh(character)
            else
                d.day = d.day + 1
                _M.persist(character)
            end
        end        
    end)

    character.monitor("simple_battle_pass", function(_, id)
        local d = _M.assert_get(character)
        if not d.open then
            local conf = assert(asset.DataConfig_proto[11], "DataConfig_proto[11]")
            if id >= conf.data1 then
                THIS.data_refresh(character)
            end
        end        
    end)
end

-- TODO: 类似泰利的 prepare 接口
function MODULE.launch(character)
    local d = _M.assert_get(character)
    local u = _M.assert_runtime(character)
end

-- TODO: 与客户端同步数据
function MODULE.ready(character)
    local d = _M.assert_get(character)
    logger.test("%s:ready, %s", module_name, stringify(d or {}))
end

-- TODO: 玩家下线时的处理
function MODULE.saybye(character)

end

function THIS.data_refresh(character)
    local d = _M.assert_get(character)
    if not d.open then
        d.round = 1
    else
        d.round = d.round + 1
    end
    d.open = true
    d.day = 1
    d.award = 0
    _M.persist(character)
    THIS.send_client(character)
end

function THIS.sign_in_award(character, args)
    local d = _M.assert_get(character) or {}
    if not d.open then
        return 1 -- 参数异常
    end
    local award_list = {}
    local rec_flag = 0
    for day = d.day, 1, -1 do
        local flag = 1 << day-1
        if flag & d.award == 0 then
            rec_flag = rec_flag + flag
            local conf_list = assert(asset.SigninConfig_proto, "SigninConfig_proto")
            local max_round = math.floor(#conf_list/MAX_DAY)
            local id = (math.min(d.round, max_round)-1)*MAX_DAY+day
            local conf = conf_list[id] 
            if not conf then
                return STD_ERR.COMMON_CONF_ERR -- 配置异常
            end

            for i = 1, #conf.reward, 2 do
                local item_id = conf.reward[i]
                local item_num = conf.reward[i+1]
                if item_id and item_num and item_id > 0 and item_num > 0 then
                    table.insert(award_list, {id = item_id, num = item_num})
                end
            end
        end
    end

    if rec_flag == 0 then
        return 1 -- 参数异常
    end

    if not next(award_list) then
        return STD_ERR.COMMON_CONF_ERR -- 配置异常
    end

    local desc = {
        module="sign_in",
        brief="签到",
        context= string.format("签到, round:%d, rec_flag:%d", d.round, rec_flag),
        mailgen={subject=2, body=""},
        notify={
            flags="sign_in_award"
        },
        detail=award_list
    }
    reward(character, desc)

    d.award = d.award + rec_flag
    _M.persist(character)
    return 0, {day = d.day, award = d.award}
end


-- function THIS.sign_in_award(character, args)
--     local day = args.day
--     if not day or day > MAX_DAY then
--         return STD_ERR.COMMON_PARM_ERR -- 参数异常
--     end

--     local d = _M.assert_get(character)
--     if not d.open then
--         return STD_ERR.COMMON_PARM_ERR -- 参数异常
--     end
    
--     if day > d.day then
--         return STD_ERR.COMMON_PARM_ERR -- 参数异常
--     end

--     local flag = 1 << day-1
--     if flag & d.award >= 1 then
--         return STD_ERR.COMMON_AWARD_RECEIVED -- 奖励已领取
--     end
--     local conf_list = assert(asset.SigninConfig_proto, "SigninConfig_proto")
--     local max_round = math.floor(#conf_list/MAX_DAY)
--     local id = (math.min(d.round, max_round)-1)*MAX_DAY+day
--     local conf = conf_list[id] 
--     if not conf then
--         return STD_ERR.COMMON_CONF_ERR -- 配置异常
--     end

--     local award_list = {}
--     for i = 1, #conf.reward, 2 do
--         local item_id = conf.reward[i]
--         local item_num = conf.reward[i+1]
--         if item_id and item_num and item_id > 0 and item_num > 0 then
--             table.insert(award_list, {id = item_id, num = item_num})
--         end
--     end

--     if not next(award_list) then
--         return STD_ERR.COMMON_CONF_ERR -- 配置异常
--     end

--     local desc = {
--         module="sign_in",
--         brief="签到",
--         context= string.format("签到, round:%d, day:%d", d.round, d.day),
--         mailgen={subject=2, body=""},
--         notify={
--             flags="sign_in_award"
--         },
--         detail=award_list
--     }
--     reward(character, desc)

--     d.award = d.award + flag
--     _M.persist(character)
--     return 0, {day = args.day}
-- end

function THIS.send_client(character)
    local d = _M.assert_get(character) or {}
    character.send("sign_in_nty", {open = d.open, round = d.round, day = d.day, award = d.award})
end

function THIS.sign_in_get_data(character, args)
    -- local d = _M.assert_get(character)
    -- return 0, {open = d.open, round = d.round, day = d.day, award = d.award}
    THIS.send_client(character)
    return 0
end

function REQUEST.sign_in_award(character, args)
    return func_ret("sign_in_award", character, args)
end

function REQUEST.sign_in_get_data(character, args)
    return func_ret("sign_in_get_data", character, args)
end

function MODULE.get_red_point(character)
    local d = _M.assert_get(character) or {}
    if d.open then
        local flag = 1<<d.day-1
        if d.award < flag then
            return true, module_id
        end
    end
end

return MODULE