local skynet = require "skynet"
require "skynet.manager"
local sharedata = require "skynet.sharedata"
local logger = require "logger"
local cache = require "skynet.codecache"
cache.mode "OFF"	-- turn off codecache, because CMD.new may load data file

local index = ...
local SET_FILES

local path = skynet.getenv "asset"
local skynet_retpack = skynet.retpack
local FILES = {
  -- TODO: 在下面定义需要引入的配置文件
    DataConfig_proto = "DataConfig_proto",              -- 离散表
    GoodsConfig_proto = "GoodsConfig_proto",            -- 货币表
    BuildingConfig_proto = "BuildingConfig_proto",      -- 建筑表
    StageConfig_proto = "StageConfig_proto",            -- 关卡表
    StageInfoConfig_proto = "StageInfoConfig_proto",    -- 关卡信息表
    EliteStageConfig_proto = "EliteStageConfig_proto",  -- 精英关卡表
    EliteInfoConfig_proto = "EliteInfoConfig_proto",    -- 精英关卡信息表
    RoleConfig_proto = "RoleConfig_proto",              -- 角色表
    RoleQualityConfig_proto = "RoleQualityConfig_proto",-- 角色升阶表
    RoleLevelConfig_proto = "RoleLevelConfig_proto",    -- 角色升级表
    CardSkillConfig_proto = "CardSkillConfig_proto",    -- 卡牌技能表
    CardLevelConfig_proto = "CardLevelConfig_proto",    -- 卡牌升级表
    CardConsumeConfig_proto = "CardConsumeConfig_proto", -- 卡牌碎片表
    RanksLevelConfig_proto = "RanksLevelConfig_proto",  -- 队伍升级表
    ArmorConfig_proto = "ArmorConfig_proto",            -- 装备表
    EquipmentQualityConfig_proto = "EquipmentQualityConfig_proto", -- 装备升阶表
    EquipmentLevelConfig_proto = "EquipmentLevelConfig_proto",  -- 装备升级表
    AwardConfig_proto   = "AwardConfig_proto",          -- 奖励库id
    TalentConfig_proto = "TalentConfig_proto",          -- 天赋配置表
    SigninConfig_proto = "SigninConfig_proto",          -- 签到配置
    TaskConfig_proto = "TaskConfig_proto",              -- 任务表
    ActiverewardConfig_proto = "ActiverewardConfig_proto", -- 活跃宝箱
    ManualConfig_proto = "ManualConfig_proto",          -- 战令
    GiftConfig_proto = "GiftConfig_proto",              -- 计费点礼包配置
    RechargeConfig_proto = "RechargeConfig_proto",      -- 充值计费点
    CallawardConfig_proto = "CallawardConfig_proto",    -- 召唤奖励库
    HeroescallConfig_proto = "HeroescallConfig_proto",  -- 英雄召唤配置
    IntegralshopConfig_proto = "IntegralshopConfig_proto", -- 积分商店
    ItemshopConfig_proto = "ItemshopConfig_proto",      -- 每日商店
    ReliclevelsConfig_proto = "ReliclevelsConfig_proto", -- 遗迹关卡表
    RelicConfig_proto = "RelicConfig_proto",            -- 遗迹信息表
    RunecardConfig_proto = "RunecardConfig_proto",      -- 遗迹符文表
    RelicmanualConfig_proto = "RelicmanualConfig_proto", -- 遗迹战令
    ActivityConfig_proto = "ActivityConfig_proto",      -- 活动配置
    DailyConfig_proto = "DailyConfig_proto",            -- 日常副本
    ChallengeConfig_proto = "ChallengeConfig_proto",    -- 挑战关卡表
    MonsterConfig_proto = "MonsterConfig_proto",        -- 怪物表
    LimitgiftConfig_proto = "LimitgiftConfig_proto",    -- 限时礼包
    FundConfig_proto = "FundConfig_proto",              -- 基金活动配置
    NewawardConfig_proto = "NewawardConfig_proto",      -- 奖励库活动
    EntryConfig_proto = "EntryConfig_proto",            -- 词条表
    CastleSkillConfig_proto = "CastleSkillConfig_proto",-- 城堡技能表
    LotteryConfig_proto = "LotteryConfig_proto",        -- 千抽赠礼配置
    TrialConfig_proto = "TrialConfig_proto",            -- 试玩配置
    I18nConfig_proto = "I18nConfig_proto",              -- 多语言表
}


-- TODO: 集群使用的配置表
local SPAN_FILES = {
    
}

local function encode(filename)
  return string.format("@%s%s.lua", path, filename)
end

local function reload(k, filename)
  local ok = pcall(sharedata.update, k, encode(filename))
  if ok then
    logger.warn("hotfix: '%s.lua'", k)
  else
    return filename
  end
end

local function hotfix(...)
  local args = { ... }
  if #args == 0 then
    for k, v in pairs(SET_FILES) do
      local error = reload(k, v)
      if error then
        return error
      end
    end
  else
    for _, k in ipairs(args) do
      local v = SET_FILES[k]
      if v then
        local error = reload(k, v)
        if error then
          return error
        end
      end
    end
  end
end

local CMD = {}
function CMD.start()
  logger.info("start")

  if index == "master" then
    SET_FILES = SPAN_FILES
  else
    SET_FILES = FILES
  end
  for k, filename in pairs(SET_FILES) do
    sharedata.new(k, encode(filename))
  end
end

function CMD.hotfix(...)
  local error = hotfix(...)
  collectgarbage "collect"
  return error
end

skynet.memlimit(128 * 1024 * 1024)
skynet.init(function() skynet.register(".assetcenter") end)
skynet.start(function()
  logger.label("<Assetcenter>")
  skynet.dispatch("lua", function(session, _, cmd, ...)
    local f = assert(CMD[cmd])
    if session == 0 then
      f(...)
    else
      skynet_retpack(f(...))
    end
  end)
end)

